@@ -1,106 +1,111 @@
 package Workflow::Action::InputField;
 
-# $Id: InputField.pm 316 2007-07-04 19:17:50Z jonasbn $
+# $Id: InputField.pm 493 2009-10-05 13:08:14Z jonasbn $
 
+use warnings;
 use strict;
 use base qw( Class::Accessor );
-use Log::Log4perl       qw( get_logger );
+use Log::Log4perl qw( get_logger );
 use Workflow::Exception qw( configuration_error );
+use English qw( -no_match_vars );
 
 $Workflow::Action::InputField::VERSION = '1.09';
 
-my @FIELDS = qw( name label description type requirement source_class source_list );
-__PACKAGE__->mk_accessors( @FIELDS );
+my @PROPS = qw( name label description type requirement
+                source_class source_list class );
+__PACKAGE__->mk_accessors(@PROPS);
 
 my %INCLUDED = ();
 
 sub new {
     my ( $class, $params ) = @_;
     my $log = get_logger();
-    $log->debug( "Instantiating new field '$params->{name}'" );
+    $log->debug("Instantiating new field '$params->{name}'")
+        if $params->{name};
 
-    my $self = bless( {}, $class );
+    my $self = bless {}, $class;
 
     # Set all our parameters
-    foreach my $field ( @FIELDS ) {
-        next unless ( $params->{ $field } );
-        $self->$field( $params->{ $field } );
+    foreach my $prop (@PROPS) {
+        next unless ( $params->{$prop} );
+        $self->$prop( $params->{$prop} );
     }
 
     # ...ensure our name is defined
     unless ( $self->name ) {
-        my $id_string = '[' .
-                        join( '] [', map { "$_: $params->{$_}" }
-                                         sort keys %{ $params } ) .
-                        ']';
+        my $id_string = '['
+            . join(
+            '] [', map {"$_: $params->{$_}"}
+                sort keys %{$params}
+            ) . ']';
         configuration_error "Field found without name: $id_string";
     }
 
     my $name = $self->name;
     unless ( $self->label ) {
-        $self->label( $name );
+        $self->label($name);
     }
-    my $requirement = ( defined $params->{is_required} && $params->{is_required} eq 'yes' )
-                        ? 'required' : 'optional';
-    $self->requirement( $requirement );
+    my $requirement = ( defined $params->{is_required}
+            && $params->{is_required} eq 'yes' ) ? 'required' : 'optional';
+    $self->requirement($requirement);
 
     # ...ensure a class associated with the input source exists
     if ( my $source_class = $self->source_class ) {
-        $log->debug( "Possible values for '$name' from '$source_class'" );
-        unless ( $INCLUDED{ $source_class } ) {
+        $log->debug("Possible values for '$name' from '$source_class'");
+        unless ( $INCLUDED{$source_class} ) {
             eval "require $source_class";
-            if ( $@ ) {
+            if ($EVAL_ERROR) {
                 configuration_error "Failed to include source class ",
-                                    "'$source_class' used in field '$name'";
+                    "'$source_class' used in field '$name'";
             }
-            $INCLUDED{ $source_class }++;
+            $INCLUDED{$source_class}++;
         }
-        $params->{values} = [ $source_class->get_possible_values( $self ) ];
-    }
-    elsif ( $self->source_list ) {
-        $log->debug( "Possible values for '$name' specified in config" );
-        $params->{values} = [ split( /\s*,\s*/, $self->source_list ) ];
+        $params->{values} = [ $source_class->get_possible_values($self) ];
+    } elsif ( $self->source_list ) {
+        $log->debug("Possible values for '$name' specified in config");
+        $params->{values} = [ split /\s*,\s*/, $self->source_list ];
     }
 
     my $values = $params->{values} || $params->{possible_values};
-    if ( $values ) {
-        my @add_values = ( ref $values eq 'ARRAY' ) ? @{ $values } : ( $values );
+    if ($values) {
+        my @add_values = ( ref $values eq 'ARRAY' ) ? @{$values} : ($values);
         $log->debug( "Values to use as source for field '$name': ",
-                     join( ', ', @add_values ) );
-        $self->add_possible_values( @add_values );
+            join ', ', @add_values );
+        $self->add_possible_values(@add_values);
     }
 
     # Assign the default field type, subclasses may override...
-    $self->type( 'basic' );
+    $self->type('basic');
 
-    $self->init( $params );
+    $self->init($params);
     return $self;
 }
 
-sub init { return }
+sub init {return}
 
 sub is_required {
-    my ( $self ) = @_;
+    my ($self) = @_;
     return ( $self->requirement eq 'required' ) ? 'yes' : 'no';
 }
 
 sub is_optional {
-    my ( $self ) = @_;
+    my ($self) = @_;
     return ( $self->requirement eq 'optional' ) ? 'yes' : 'no';
 }
 
 sub get_possible_values {
-    my ( $self ) = @_;
+    my ($self) = @_;
     $self->{_enumerated} ||= [];
     return @{ $self->{_enumerated} };
 }
 
 sub add_possible_values {
     my ( $self, @values ) = @_;
-    foreach my $value ( @values ) {
-        my $this_value = ( ref $value eq 'HASH' )
-                           ? $value
-                           : { label => $value, value => $value };
+    foreach my $value (@values) {
+        my $this_value =
+            ( ref $value eq 'HASH' )
+            ? $value
+            : { label => $value, value => $value };
         push @{ $self->{_enumerated} }, $this_value;
     }
     return @{ $self->{_enumerated} };
@@ -114,6 +119,10 @@ __END__
 
 Workflow::Action::InputField - Metadata about information required by an Action
 
+=head1 VERSION
+
+This documentation describes version 1.09 of this package
+
 =head1 SYNOPSIS
 
  # Declare the fields needed by your action in the configuration...
@@ -171,6 +180,54 @@ Typical constructor; will throw exception if 'name' is not defined or
 if the property 'source_class' is defined but the class it specifies
 is not available.
 
+You will usually not need to use or override this method unless you
+derive your own input field class (see I<class> in L</"Properties">
+below). For example, suppose you need to add extra properties to all
+your fields like "index", "disabled", etc.
+
+In your actions definition XML file, you can just add them and the
+parser will pick them up. Pay close attention the custom InputField
+"class" property.
+
+  <actions>
+    <type>foo</type>
+    <action name="Bar"
+      class="your::action::class">
+      <field index="0" name="id" type="integer" disabled="yes"
+        is_required="yes" class="your::custom::inputfieldclass"/>
+    </action>
+
+But you need to give them life by creating the accessors for these
+extra properties. Just derive your custom fields class like so:
+
+  package your::custom::inputfieldclass;
+
+  use warnings;
+  use strict;
+
+  use base qw( Workflow::Action::InputField );
+  use Workflow::Exception qw( workflow_error );
+
+  # extra action class properties
+  my @EXTRA_PROPS = qw( index disabled );
+  __PACKAGE__->mk_accessors(@EXTRA_PROPS);
+
+  sub new {
+    my ( $class, $params ) = @_;
+    my $self = $class->SUPER::new($params);
+    # set only our extra properties
+    foreach my $prop (@EXTRA_PROPS) {
+      next if ( $self->$prop );
+      $self->$prop( $params->{$prop} );
+    }
+    warn "INDEX IS NOW WORKING:".$self->index;
+    warn "AND SO IS DISABLED:".$self->disabled;
+    return $self;
+  }
+
+  1;
+
+
 =head3 is_required()
 
 Returns 'yes' if field is required, 'no' if optional.
@@ -214,8 +271,11 @@ you the information without much fuss.
 
 B<type> (optional)
 
-TODO: Datatype of field (still under construction...). By default it
-is set to 'basic'.
+Field types are implementation dependant are they should be
+intrinsically implemented by validators. In other words, you can use
+any mnemonic value for your convinience like "integer", "text",
+etc. but it won't affect anything unless you use a validator to
+validate your action data. By default it is set to 'basic'.
 
 B<requirement> ('required'|'optional')
 
@@ -234,6 +294,13 @@ possible values for the field. The resulting list returned from
 C<get_possible_values()> will have the same value for both the 'label'
 and 'value' keys.
 
+B<class> (optional)
+
+You may specify a custom InputField class. It should C<use base qw(
+Workflow::Action );> and probably override the new() method which
+should call SUPER::new($params). See L</"new( \%params )"> above for an
+example.
+
 =head1 SEE ALSO
 
 L<Workflow::Action>
